@@ -254,7 +254,7 @@ renderUserCommandBox st hs =
                 in hBox [ replyArrow
                         , addEllipsis $ renderMessage MessageData
                           { mdMessage           = msgWithoutParent
-                          , mdUserName          = msgWithoutParent^.mUser.to (nameForUserRef st)
+                          , mdUserName          = msgWithoutParent^.mUser.to (printableNameForUserRef st)
                           , mdParentMessage     = Nothing
                           , mdParentUserName    = Nothing
                           , mdHighlightSet      = hs
@@ -266,7 +266,9 @@ renderUserCommandBox st hs =
                           , mdShowReactions     = True
                           , mdMessageWidthLimit = Nothing
                           , mdMyUsername        = myUsername st
+                          , mdMyUserId          = myUserId st
                           , mdWrapNonhighlightedCodeBlocks = True
+                          , mdTruncateVerbatimBlocks = Nothing
                           }
                         ]
             _ -> emptyWidget
@@ -373,8 +375,8 @@ renderCurrentChannelDisplay st hs = header <=> hBorder <=> messages
 
             let maxHeight = max (Vty.imageHeight $ statusBox^.imageL)
                                 (Vty.imageHeight $ channelHeaderResult^.imageL)
-                statusBoxWidget = Widget Fixed Fixed $ return statusBox
-                headerWidget = Widget Fixed Fixed $ return channelHeaderResult
+                statusBoxWidget = resultToWidget statusBox
+                headerWidget = resultToWidget channelHeaderResult
                 borderWidget = vLimit maxHeight vBorder
 
             render $ if st^.csCurrentTeam.tsMode == ChannelSelect
@@ -710,7 +712,7 @@ maybePreviewViewport tId w =
             False -> return result
             True ->
                 render $ vLimit previewMaxHeight $ viewport (MessagePreviewViewport tId) Vertical $
-                         (Widget Fixed Fixed $ return result)
+                         (resultToWidget result)
 
 inputPreview :: ChatState -> HighlightSet -> Widget Name
 inputPreview st hs | not $ st^.csResources.crConfiguration.configShowMessagePreviewL = emptyWidget
@@ -743,9 +745,9 @@ inputPreview st hs | not $ st^.csResources.crConfiguration.configShowMessagePrev
                                   else prview pm $ getParentMessage st pm
                      prview m p = renderMessage MessageData
                                   { mdMessage           = m
-                                  , mdUserName          = m^.mUser.to (nameForUserRef st)
+                                  , mdUserName          = m^.mUser.to (printableNameForUserRef st)
                                   , mdParentMessage     = p
-                                  , mdParentUserName    = p >>= (^.mUser.to (nameForUserRef st))
+                                  , mdParentUserName    = p >>= (^.mUser.to (printableNameForUserRef st))
                                   , mdHighlightSet      = hs
                                   , mdEditThreshold     = Nothing
                                   , mdShowOlderEdits    = False
@@ -755,7 +757,9 @@ inputPreview st hs | not $ st^.csResources.crConfiguration.configShowMessagePrev
                                   , mdShowReactions     = True
                                   , mdMessageWidthLimit = Nothing
                                   , mdMyUsername        = myUsername st
+                                  , mdMyUserId          = myUserId st
                                   , mdWrapNonhighlightedCodeBlocks = True
+                                  , mdTruncateVerbatimBlocks = Nothing
                                   }
                  in (maybePreviewViewport tId msgPreview) <=>
                     hBorderWithLabel (withDefAttr clientEmphAttr $ str "[Preview ↑]")
@@ -837,10 +841,10 @@ mainInterface st =
         in case allTypingUsers (st^.csCurrentChannel.ccInfo.cdTypingUsers) of
             [] -> emptyWidget
             [uId] | Just un <- usernameForUserId uId st ->
-               format $ "[" <> userSigil <> un <> " is typing]"
+               format $ "[" <> addUserSigil un <> " is typing]"
             [uId1, uId2] | Just un1 <- usernameForUserId uId1 st
                          , Just un2 <- usernameForUserId uId2 st ->
-               format $ "[" <> userSigil <> un1 <> " and " <> userSigil <> un2 <> " are typing]"
+               format $ "[" <> addUserSigil un1 <> " and " <> addUserSigil un2 <> " are typing]"
             _ -> format "[several people are typing]"
 
     showBusy = case st^.csWorkerIsBusy of
