@@ -87,7 +87,7 @@ import           Matterhorn.Types ( ConnectionInfo(..)
                        , ciPassword, ciUsername, ciHostname, ciUrlPath
                        , ciPort, ciType, AuthenticationException(..)
                        , LogManager, LogCategory(..), ioLogWithManager
-                       , ciAccessToken
+                       , ciAccessToken, SemEq(..)
                        )
 
 
@@ -99,6 +99,9 @@ data Name =
     | AccessToken
     deriving (Ord, Eq, Show)
 
+instance SemEq Name where
+    semeq = (==)
+
 -- | The result of an authentication attempt.
 data LoginAttempt =
     AttemptFailed AuthenticationException
