@@ -54,6 +54,8 @@ module Matterhorn.Types
   , channelTopicDialogEditor
   , channelTopicDialogFocus
 
+  , resultToWidget
+
   , newSaveAttachmentDialog
   , SaveAttachmentDialogState(..)
   , attachmentPathEditor
@@ -78,6 +80,7 @@ module Matterhorn.Types
   , configActivityNotifyCommandL
   , configActivityNotifyVersionL
   , configActivityBellL
+  , configTruncateVerbatimBlocksL
   , configShowMessageTimestampsL
   , configShowBackgroundL
   , configShowMessagePreviewL
@@ -124,7 +127,7 @@ module Matterhorn.Types
   , teamZipperIds
   , mkChannelZipperList
   , ChannelListGroup(..)
-  , channelListGroupUnread
+  , ChannelListGroupLabel(..)
   , nonDMChannelListGroupUnread
 
   , trimChannelSigil
@@ -166,6 +169,7 @@ module Matterhorn.Types
   , csChannelListOrientation
   , csResources
   , csLastMouseDownEvent
+  , csVerbatimTruncateSetting
   , csCurrentChannel
   , csCurrentChannelId
   , csCurrentTeamId
@@ -289,6 +293,7 @@ module Matterhorn.Types
   , mhSuspendAndResume
   , mhHandleEventLensed
   , mhHandleEventLensed'
+  , mhContinueWithoutRedraw
   , St.gets
   , mhError
 
@@ -361,7 +366,7 @@ import           Matterhorn.Prelude
 
 import qualified Graphics.Vty as Vty
 import qualified Brick
-import           Brick ( EventM, Next, Widget )
+import           Brick ( EventM, Next, Widget(..), Size(..), Result )
 import           Brick.Focus ( FocusRing, focusRing )
 import           Brick.Themes ( Theme )
 import           Brick.Main ( invalidateCache, invalidateCacheEntry )
@@ -410,7 +415,7 @@ import           Network.Mattermost.Types
 import           Network.Mattermost.Types.Config
 import           Network.Mattermost.WebSocket ( WebsocketEvent, WebsocketActionResponse )
 
-import           Matterhorn.Constants ( userSigil, normalChannelSigil )
+import           Matterhorn.Constants ( normalChannelSigil )
 import           Matterhorn.InputHistory
 import           Matterhorn.Emoji
 import           Matterhorn.Types.Common
@@ -448,24 +453,23 @@ data TokenSource =
 -- | The type of channel list group headings. Integer arguments indicate
 -- total number of channels in the group that have unread activity.
 data ChannelListGroup =
-    ChannelGroupPublicChannels Int
-    | ChannelGroupPrivateChannels Int
-    | ChannelGroupFavoriteChannels Int
-    | ChannelGroupDirectMessages Int
+    ChannelListGroup { channelListGroupLabel :: ChannelListGroupLabel
+                     , channelListGroupUnread :: Int
+                     }
+                     deriving (Eq)
+
+data ChannelListGroupLabel =
+    ChannelGroupPublicChannels
+    | ChannelGroupPrivateChannels
+    | ChannelGroupFavoriteChannels
+    | ChannelGroupDirectMessages
     deriving (Eq)
 
-channelListGroupUnread :: ChannelListGroup -> Int
-channelListGroupUnread (ChannelGroupPublicChannels n)  = n
-channelListGroupUnread (ChannelGroupPrivateChannels n) = n
-channelListGroupUnread (ChannelGroupFavoriteChannels n) = n
-channelListGroupUnread (ChannelGroupDirectMessages n)  = n
-
-
 nonDMChannelListGroupUnread :: ChannelListGroup -> Int
-nonDMChannelListGroupUnread (ChannelGroupPublicChannels n)  = n
-nonDMChannelListGroupUnread (ChannelGroupPrivateChannels n) = n
-nonDMChannelListGroupUnread (ChannelGroupFavoriteChannels n) = n
-nonDMChannelListGroupUnread (ChannelGroupDirectMessages _)  = 0
+nonDMChannelListGroupUnread g =
+    case channelListGroupLabel g of
+        ChannelGroupDirectMessages -> 0
+        _ -> channelListGroupUnread g
 
 -- | The type of channel list entries.
 data ChannelListEntry =
@@ -474,6 +478,7 @@ data ChannelListEntry =
                      , channelListEntryUnread :: Bool
                      , channelListEntrySortValue :: T.Text
                      , channelListEntryFavorite :: Bool
+                     , channelListEntryMuted :: Bool
                      }
                      deriving (Eq, Show, Ord)
 
@@ -527,6 +532,9 @@ data Config =
            -- ^ The activity notifier version.
            , configActivityBell :: Bool
            -- ^ Whether to ring the terminal bell on activity.
+           , configTruncateVerbatimBlocks :: Maybe Int
+           -- ^ Whether to truncate verbatim (and code) blocks past a
+           -- reasonable number of lines.
            , configShowMessageTimestamps :: Bool
            -- ^ Whether to show timestamps on messages.
            , configShowBackground :: BackgroundInfo
@@ -636,17 +644,17 @@ mkChannelZipperList now config tId cconfig prefs cs us =
         (dmFavs,   dmEntries)   = partitionFavorites $ getDMChannelEntries now config cconfig prefs us cs
         favEntries              = privFavs <> normFavs <> dmFavs
     in [ let unread = length $ filter channelListEntryUnread favEntries
-         in (ChannelGroupFavoriteChannels unread, sortChannelListEntries favEntries)
+         in (ChannelListGroup ChannelGroupFavoriteChannels unread, sortChannelListEntries favEntries)
        , let unread = length $ filter channelListEntryUnread normEntries
-         in (ChannelGroupPublicChannels unread, sortChannelListEntries normEntries)
+         in (ChannelListGroup ChannelGroupPublicChannels unread, sortChannelListEntries normEntries)
        , let unread = length $ filter channelListEntryUnread privEntries
-         in (ChannelGroupPrivateChannels unread, sortChannelListEntries privEntries)
+         in (ChannelListGroup ChannelGroupPrivateChannels unread, sortChannelListEntries privEntries)
        , let unread = length $ filter channelListEntryUnread dmEntries
-         in (ChannelGroupDirectMessages unread, sortDMChannelListEntries dmEntries)
+         in (ChannelListGroup ChannelGroupDirectMessages unread, sortDMChannelListEntries dmEntries)
        ]
 
 sortChannelListEntries :: [ChannelListEntry] -> [ChannelListEntry]
-sortChannelListEntries = sortBy (comparing channelListEntrySortValue)
+sortChannelListEntries = sortBy (comparing (\c -> (channelListEntryMuted c, channelListEntrySortValue c)))
 
 sortDMChannelListEntries :: [ChannelListEntry] -> [ChannelListEntry]
 sortDMChannelListEntries = sortBy compareDMChannelListEntries
@@ -662,6 +670,7 @@ getChannelEntriesByType tId prefs cs ty =
         entries = mkEntry <$> pairs
         mkEntry (cId, ch) = ChannelListEntry { channelListEntryChannelId = cId
                                              , channelListEntryType = CLChannel
+                                             , channelListEntryMuted = isMuted ch
                                              , channelListEntryUnread = hasUnread' ch
                                              , channelListEntrySortValue = ch^.ccInfo.cdDisplayName.to T.toLower
                                              , channelListEntryFavorite = isFavorite prefs cId
@@ -719,6 +728,7 @@ getGroupDMChannelEntries now config prefs cs =
                             groupChannelShouldAppear now config prefs info
     in fmap (\(cId, ch) -> ChannelListEntry { channelListEntryChannelId = cId
                                             , channelListEntryType = CLGroupDM
+                                            , channelListEntryMuted = isMuted ch
                                             , channelListEntryUnread = hasUnread' ch
                                             , channelListEntrySortValue = ch^.ccInfo.cdDisplayName
                                             , channelListEntryFavorite = isFavorite prefs cId
@@ -744,6 +754,7 @@ getSingleDMChannelEntries now config cconfig prefs us cs =
                     if dmChannelShouldAppear now config prefs c
                     then return (ChannelListEntry { channelListEntryChannelId = cId
                                                   , channelListEntryType = CLUserDM uId
+                                                  , channelListEntryMuted = isMuted c
                                                   , channelListEntryUnread = hasUnread' c
                                                   , channelListEntrySortValue = displayNameForUser u cconfig prefs
                                                   , channelListEntryFavorite = isFavorite prefs cId
@@ -873,6 +884,7 @@ data Name =
     | ClickableURLListEntry Int LinkTarget
     | ClickableReactionInMessage PostId Text (Set UserId)
     | ClickableReaction PostId Text (Set UserId)
+    | ClickableAttachment FileId
     | AttachmentPathEditor TeamId
     | AttachmentPathSaveButton TeamId
     | AttachmentPathCancelButton TeamId
@@ -1162,9 +1174,9 @@ autocompleteAlternativeReplacement :: AutocompleteAlternative -> Text
 autocompleteAlternativeReplacement (EmojiCompletion e) =
     ":" <> e <> ":"
 autocompleteAlternativeReplacement (SpecialMention m) =
-    userSigil <> specialMentionName m
+    addUserSigil $ specialMentionName m
 autocompleteAlternativeReplacement (UserCompletion u _) =
-    userSigil <> userUsername u
+    addUserSigil $ userUsername u
 autocompleteAlternativeReplacement (ChannelCompletion _ c) =
     normalChannelSigil <> (sanitizeUserText $ channelName c)
 autocompleteAlternativeReplacement (SyntaxCompletion t) =
@@ -1508,6 +1520,10 @@ data ChatState =
               -- ^ The most recent mouse click event we got. We reset
               -- this on mouse up so we can ignore clicks whenever this
               -- is already set.
+              , _csVerbatimTruncateSetting :: Maybe Int
+              -- ^ The current verbatim block truncation setting. This
+              -- is used to toggle truncation behavior and is updated
+              -- from the configTruncateVerbatimBlocks Config field.
               , _csTeams :: HashMap TeamId TeamState
               -- ^ The state for each team that we are in.
               , _csTeamZipper :: Z.Zipper () TeamId
@@ -1973,6 +1989,11 @@ mhSuspendAndResume mote = MH $ do
     s <- St.get
     St.put $ s { mhNextAction = \ _ -> Brick.suspendAndResume (mote $ mhCurrentState s) }
 
+mhContinueWithoutRedraw :: MH ()
+mhContinueWithoutRedraw = MH $ do
+    s <- St.get
+    St.put $ s { mhNextAction = \ _ -> Brick.continueWithoutRedraw (mhCurrentState s) }
+
 -- | This will request that after this computation finishes the
 -- application should exit
 requestQuit :: MH ()
@@ -2125,6 +2146,7 @@ newState (StartupStateInfo {..}) =
     in applyTeamOrderPref (_userPrefTeamOrder $ _crUserPreferences startupStateResources) $
        ChatState { _csResources                   = startupStateResources
                  , _csLastMouseDownEvent          = Nothing
+                 , _csVerbatimTruncateSetting     = configTruncateVerbatimBlocks config
                  , _csTeamZipper                  = Z.findRight (== startupStateInitialTeam) $
                                                     mkTeamZipper startupStateTeams
                  , _csTeams                       = startupStateTeams
@@ -2493,3 +2515,6 @@ moveRight v as =
             | otherwise ->
                 let (h, t) = splitAt i as
                 in h <> [head (tail t), v] <> (tail (tail t))
+
+resultToWidget :: Result n -> Widget n
+resultToWidget = Widget Fixed Fixed . return
